#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ChemistryParadise::InteractiveChemistryShell
#
# Usage examples:
#   ChemistryParadise::InteractiveChemistryShell.new
# =========================================================================== #
require 'chemistry_paradise/base/base.rb'
require 'chemistry_paradise/calculate_atomic_mass.rb'

begin
  require 'readline'
rescue LoadError; end

module ChemistryParadise

class InteractiveChemistryShell < Base # require 'chemistry/interactive_chemistry_shell'; InteractiveChemistryShell.new

  # ========================================================================= #
  # === PROMPT
  # ========================================================================= #
  PROMPT = '> '
  if File.exist? Shared.periodic_table?
    PERIODIC_TABLE = YAML.load_file(Shared.periodic_table?)
  else
    PERIODIC_TABLE = nil
  end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(run_already = true)
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @lpad = '  '
  end

  # ========================================================================= #
  # === show_welcome_message
  # ========================================================================= #
  def show_welcome_message
    e 'Welcome to the InteractiveChemistryShell.'
    e 'Input "help" to get a list of options, otherwise'
    e 'just type in your input now.'
    print PROMPT
  end

  # ========================================================================= #
  # === report_input
  # ========================================================================= #
  def report_input(i)
    ewarn 'Input was: `'+sfancy(i)+'`'
  end

  # ========================================================================= #
  # === show_welcome_message_and_help
  # ========================================================================= #
  def show_welcome_message_and_help
    show_welcome_message
    show_help
  end

  # ========================================================================= #
  # === calculate
  #
  # To test this, do:
  #   calc C3H5
  # ========================================================================= #
  def calculate(i)
    if i.nil?
      e 'Please provide proper input to this method now, for instance, `C12H12N2`:'
      i = $stdin.gets.chomp
    end
    CalculateAtomicMass.new(i)
  end; alias calc calculate # === calc

  # ========================================================================= #
  # === check_against_menu
  # ========================================================================= #
  def check_against_menu
    _ = @user_input # We can use this now.
    arguments = nil # Default to denote that we have no arguments.
    if _.include? ' '
      splitted = _.split(' ')
      _ = splitted[0] # Reassign here to the first entry.
      arguments = splitted[1..-1]
    end
    case _ # case tag
    # ======================================================================= #
    # === quit
    # ======================================================================= #
    when 'quit','exit','q'
      e 'Bye from the InteractiveChemistryShell.'
      exit
    # ======================================================================= #
    # === show_electron_negativity_chart
    # ======================================================================= #
    when '4','show_electron_negativity_chart'
      ChemistryParadise.show_electron_negativity_chart
    # ======================================================================= #
    # === periodic_table?
    # ======================================================================= #
    when '3','periodic_table?','show_periodic_table','top','top?',
         'periodic?','periodic','showperiodictable'
      show_periodic_table
    # ======================================================================= #
    # === help
    # ======================================================================= #
    when /-?-?help/,'hel','he','h','?'
      show_help
    when '1','calc','calculate'
      calculate(arguments)
    when '2','show'
      e 'Input your element symbol now:'
      element_symbol = $stdin.gets.chomp
      ShowElectronConfiguration.new(element_symbol)
    when '' # pass through
    else
      # To test the following, try:
      #   454 g NH4NO3
      if @user_input.include?('g') and @user_input =~ /\d+/
        splitted = @user_input.strip.split('g').map(&:strip)
        atomic_mass = ChemistryParadise::CalculateAtomicMass[splitted.last].to_f
        n_times = splitted.first.strip.to_i
        mass = n_times / atomic_mass
        e (mass.round(2)).to_s+' mol'
      else
        report_input(_)
        show_help
      end
    end
  end

  # ========================================================================= #
  # === ecomment
  # ========================================================================= #
  def ecomment(i)
    Colours.ecomment(i)
  end

  # ========================================================================= #
  # === run_loop
  # ========================================================================= #
  def run_loop
    loop {
      fetch_user_input # Get user input first.
      check_against_menu
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    show_welcome_message_and_help # Show the help section on startup.
    run_loop
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help # help tag
    e 'Available options:'
    e
    # ======================================================================= #
    # Add more documented help-options to this Array here.
    # ======================================================================= #
    array = [
      'calculate the atomic mass',
      'show the electron configuration of an Element',
      'show the periodic table',
      'show electron negativity chart'
    ]
    array.each_with_index {|entry, index|
      index += 1
      ecomment @lpad+index.to_s+' # '+entry
    }
  end

  # ========================================================================= #
  # === has_readline?
  # ========================================================================= #
  def has_readline?
    Object.const_defined? :Readline
  end

  # ========================================================================= #
  # === fetch_user_input
  # ========================================================================= #
  def fetch_user_input
    if has_readline?
      @user_input = Readline.readline('', true)
    else
      @user_input = $stdin.gets.chomp
    end
  end

  # ========================================================================= #
  # === show_periodic_table
  # ========================================================================= #
  def show_periodic_table
    e # Add a newline.
    inverted = PERIODIC_TABLE.invert
    1.upto(7).each {|periode|
      range = return_range_for_this_period(periode)
      # ===================================================================== #
      # Now we have a proper range.
      # ===================================================================== #
      matches = inverted.select {|index_position, value|
        range.include? index_position
      }
      ee colourize_via_kde_konsole(periode.to_s)+': ' # Display the periode here.
      matches.each_pair {|key, value|
        ee simp(value)+' ('+key.to_s+') '
      }; e
    }
  end

  # ========================================================================= #
  # === colourize_via_kde_konsole
  # ========================================================================= #
  def colourize_via_kde_konsole(i)
    if Object.const_defined? :Colours
      i = ::Colours.lightgreen(i)
    end
    i
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ChemistryParadise::InteractiveChemistryShell.new
end # ichem