#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ChemistryParadise::Shared
#
# To load this file, do:
#
#   require 'chemistry_paradise/shared.rb'
#
# =========================================================================== #
require 'chemistry_paradise/requires/common_external_requires.rb'
require 'chemistry_paradise/constants/constants.rb'
require 'chemistry_paradise/version/version.rb'
require 'chemistry_paradise/toplevel_methods/periodic_table.rb'
require 'chemistry_paradise/toplevel_methods/show_electron_negativity_chart.rb'

module ChemistryParadise # include ChemistryParadise::Shared

module Shared # === ChemistryParadise::Shared

  include ChemistryParadise::Constants

  # ========================================================================= #
  # === ARRAY_TEST_THESE_MOLECULES
  #
  # Test these molecules. Default values.
  # ========================================================================= #
  ARRAY_TEST_THESE_MOLECULES = %w(
    NH3
    NH4
    MnO2
    P2O5
    H2O
    Pb3O4
    H2SO4
    C12H12N2
    BaCO3
    PbCrO4
    Al2(SO4)3
    Ca(OH)2
    CH3COOH
    C12H22O11
  )

  # ========================================================================= #
  # === is_number?
  #
  # This method will return true if is a number, else false. For this it
  # uses the .to_i method trick, which returns 0 for non-numbers.
  # ========================================================================= #
  def is_number?(i)
    result = (i.to_i.to_s == i)
    return result
  end

  # ========================================================================= #
  # === square
  #
  # Die Wurzel aus 2 ist 1.414 
  #   chem; square 2; sqr 3; sqr 4; sqr 5; sqr 6; sqr 7; sqr 8; sqr 9
  # ========================================================================= #
  def square(of)
    Math.sqrt of
  end; alias sqr square # === sqr

  # ========================================================================= #
  # === ChemistryParadise::Shared.convert_parens
  #
  # This method will properly convert the () parens that can be found
  # in a chemical formula.
  #
  # For instance:
  #    Al2(SO4)3
  #
  # Is effectively the same such as:
  #    Al2SO4SO4SO4
  # ========================================================================= #
  def self.convert_parens(i)
    if i.is_a? Array
      i.each {|entry| convert_parens(entry) }
    else
      if i.include? '('
        n_parens = i.count('(')
        if n_parens > 1 # Ok, this may be a really complex formula, like:
                        #   Fe(OH)3 + H2SO4 -> Fe2(SO4)3 + H2O
          splitted = i.split(' ')
          splitted = splitted.map {|entry| convert_parens(entry) } # For now we remove some things. 
          return splitted.join(' ')
        else
          regex = /\((.+)\)(\d)/
          match = i.match(regex)
          i = i[0, i.index('(')]
          i << match[1] * match[2].to_i # Which group * n repetition
        end
      end
      return i
    end
  end

  # ========================================================================= #
  # === periodic_table?
  # ========================================================================= #
  def periodic_table?
    return Shared.periodic_table?
  end

  # ========================================================================= #
  # === ChemistryParadise::Shared.periodic_table?
  # ========================================================================= #
  def self.periodic_table?
    ::ChemistryParadise.periodic_table?
  end

  # ========================================================================= #
  # === return_range_for_this_period
  #
  # This method will tell us the legal range of elements for any given
  # period.
  #
  # The formula would be:
  #
  #   2+4n
  #
  # For instance:
  #   period 1 will return (1..2).
  #   period 2 will return (3..8).
  # ========================================================================= #
  def return_range_for_this_period(this_period = 1)
    case this_period
    when 1
      (1..2)
    when 2
      (3..10)
    when 3
      (11..18)
    when 4
      (19..36)
    when 5
      (37..54)
    when 6
      (55..86)
    when 7
      (87..118)
    end
  end

  # ========================================================================= #
  # === convert_parens
  # ========================================================================= #
  def convert_parens(i)
    ChemistryParadise::Shared.convert_parens(i)
  end; alias parse convert_parens # === parse

end; end

if __FILE__ == $PROGRAM_NAME
  include ChemistryParadise::Shared
  pp convert_parens 'Al2(SO4)3'
  pp convert_parens 'Fe(OH)3 + H2SO4 -> Fe2(SO4)3 + H2O'
  ChemistryParadise::Shared.show_electron_negativity_chart
end # rb shared.rb