#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ChemistryParadise::ShowElectronConfiguration
#
# Feed this class an element as input, such as Ca or Fe, and it will
# display the electron configuration of said element.
#
# Usage examples:
#   ChemistryParadise::ShowElectronConfiguration.new
# =========================================================================== #
# require 'chemistry_paradise/show_electron_configuration.rb'
# =========================================================================== #
require 'chemistry_paradise/base/base.rb'

module ChemistryParadise

class ShowElectronConfiguration < Base  # ShowElectronConfiguration.new

  # ========================================================================= #
  # === DEFAULT_INPUT
  # ========================================================================= #
  DEFAULT_INPUT = 'Ca'

  # ========================================================================= #
  # === NEBENQUANTENZAHL
  # ========================================================================= #
  NEBENQUANTENZAHL = {
    's' =>  2, 
    'p' =>  6,
    'd' => 10,
    'f' => 14
  }

  # ========================================================================= #
  # 1. Periode: 1s
  # 2. Periode: 2s 2p
  # 3. Periode: 3s 3p
  # 4. Periode: 4s 3d 4p
  # 5. Periode: 5s 4d 5p
  # 6. Periode: 6s 4f 5d 6p
  # 7. Periode: 7s 5f 6d
  # ========================================================================= #
  # The following hash was taken from:
  #
  #   http://en.wikipedia.org/wiki/Electron_configuration#Other_exceptions_to_Madelung.27s_rule
  #
  # and from:
  #
  #   http://en.wikipedia.org/wiki/Electron_configurations_of_the_elements_%28data_page%29
  #
  # ========================================================================= #
  MAIN_HASH = {
     1 => '1s1',
     2 => '1s2',
     3 => '1s2 2s1',
     4 => '1s2 2s2',
     5 => '1s2 2s2 2p1',         #  (5) Bor
     6 => '1s2 2s2 2p2',         #  (6) Kohlenstoff
     7 => '1s2 2s2 2p3',         #  (7) Stickstoff
     8 => '1s2 2s2 2p4',         #  (8) Sauerstoff 
     9 => '1s2 2s2 2p5',         #  (9) Fluor
    10 => '1s2 2s2 2p6',         # (10) Neon
    11 => '1s2 2s2 2p6 3s1',     # (11) Sodium
    12 => '1s2 2s2 2p6 3s2',     # (12) Magnesium
    13 => '1s2 2s2 2p6 3s2 3p1', # (13) Aluminium
    14 => '1s2 2s2 2p6 3s2 3p2', # (14) Silicon
    15 => '1s2 2s2 2p6 3s2 3p3', # (15) Phosphorus
    16 => '1s2 2s2 2p6 3s2 3p4', # (16) Sulfur
    17 => '1s2 2s2 2p6 3s2 3p5', # (17) Chlorine
    18 => '1s2 2s2 2p6 3s2 3p6', # (18) Arsen
    19 => '1s2 2s2 2p6 3s2 3p6', # (19) Potassium
    20 => '[Ar] 4s2',            # (20) Calcium
    21 => '[Ar] 4s2 3d1',        # (21) Scandium
    22 => '[Ar] 4s2 3d2',        # (22) Titanium
    23 => '[Ar] 4s2 3d3',        # (23) Vanadium
    24 => '[Ar] 4s1 3d5',        # (24) Chromium
    25 => '[Ar] 4s2 3d5',        # (25) Manganese
    26 => '[Ar] 4s2 3d6',        # (26) Iron
    27 => '[Ar] 4s2 3d7',        # (27) Cobalt
    28 => '[Ar] 4s2 3d8',        # (28) Nickel 
    29 => '[Ar] 4s1 3d10',       # (29) Copper
    30 => '[Ar] 4s2 3d10',       # (30) Zinc
    31 => '[Ar] 3d10 4s2 4p1',   # (31) Gallium
    32 => '[Ar] 3d10 4s2 4p2',   # (32) Germanium
    33 => '[Ar] 3d10 4s2 4p3',   # (33) Arsenic
    34 => '[Ar] 3d10 4s2 4p4',   # (34) Selenium
    35 => '[Ar] 3d10 4s2 4p5',   # (35) Bromine
    36 => '[Ar] 3d10 4s2 4p6',   # (36) Krypton
    37 => '[Kr] 5s1',            # (37) Rubidium
    38 => '[Kr] 5s2',            # (38) Strontium
    39 => '[Kr] 4d1 5s2',        # (39) Yttrium
    40 => '[Kr] 4d2 5s2',        # (40) Zirconium
    41 => '[Kr] 4d4 5s1',        # (41) Niobium
    42 => '[Kr] 4d5 5s1',        # (42) Molybdenum 
    43 => '[Kr] 4d5 5s2 ',       # (43) Technetium
    44 => '[Kr] 4d7 5s1',        # (44) Ruthenium
    45 => '[Kr] 4d8 5s1',        # (45) Rhodium
    46 => '[Kr] 4d10',           # (46) Palladium
    47 => '[Kr] 4d10 5s1',       # (47) Silver
    48 => '[Kr] 4d10 5s2',       # (48) Cadmium
    49 => '[Kr] 4d10 5s2 5p1',   # (49) Indium
    50 => '[Kr] 4d10 5s2 5p2',   # (50) Tin
    51 => '[Kr] 4d10 5s2 5p3',   # (51) Antimony
    52 => '[Kr] 4d10 5s2 5p4',   # (52) Tellurium
    53 => '[Kr] 4d10 5s2 5p5',   # (53) Iodine
    54 => '[Kr] 4d10 5s2 5p6',   # (54) Xenon
    55 => '[Xe] 6s1',            # (55) Caesium
  }

  _ = Shared.periodic_table?

  # ========================================================================= #
  # === PERIODIC_TABLE
  # ========================================================================= #
  PERIODIC_TABLE = YAML.load_file(_) if File.exist? _

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = nil,
      run_already    = true
    )
    reset
    set_input(optional_input)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @n_electrons = 0
    @_ = '' # The result string.
  end

  # ========================================================================= #
  # === set_main_string
  # ========================================================================= #
  def set_main_string(i)
    @_ = i
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = N)
    i = i.first if i.is_a? Array
    i = DEFAULT_INPUT if i.nil?
    i = i.to_s.dup.delete('/') # We don't need '/' characters.
    i = periodic_table?.invert[i.to_i] if i =~ /^\d+$/
    # ======================================================================= #
    # Since as of June 2016, we will upcase the first character.
    # ======================================================================= #
    i = i.dup if i.frozen?
    i[0,1] = i[0,1].upcase
    @input = i
  end

  # ========================================================================= #
  # === set_n_electrons
  # ========================================================================= #
  def set_n_electrons(i)
    @n_electrons = i
  end

  # ========================================================================= #
  # === periodic_table?
  # ========================================================================= #
  def periodic_table?
    PERIODIC_TABLE
  end

  # ========================================================================= #
  # === default_colour
  # ========================================================================= #
  def default_colour
    Colours::GREY
  end

  # ========================================================================= #
  # === main_hash?
  # ========================================================================= #
  def main_hash?
    MAIN_HASH
  end

  # ========================================================================= #
  # === check_for_inclusion
  # ========================================================================= #
  def check_for_inclusion
    if periodic_table?.has_key? @input
      set_n_electrons periodic_table?[@input]
      e 'Found element '+sfancy(@input)+'. '+
        'It has '+simp(@n_electrons.to_s)+' electrons.'
      display_electron_configuration(@n_electrons)
    else
      e "Did not find element called `#{sfancy(@input)}`."
    end
  end

  # ========================================================================= #
  # === display_electron_configuration
  #
  # Show the specific electron configuration.
  #
  # For instance:
  #   1s2 2s2 2p6 3s2 3p4
  # ========================================================================= #
  def display_electron_configuration(i = @n_electrons)
    # ======================================================================= #
    # Notify the user of invalid input.
    # ======================================================================= #
    unless main_hash?.has_key? i
      e 'Note that the electron at position '+i.to_s+
        ' is not registered.'
    end
    set_main_string MAIN_HASH[i]
    # ======================================================================= #
    # Output it next:
    # ======================================================================= #
    e yellow(@_)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_for_inclusion
  end

end; end

if __FILE__ == $PROGRAM_NAME
  include ChemistryParadise
  if ARGV.first
    ShowElectronConfiguration.new(ARGV.first)
  else
    ShowElectronConfiguration.new(ARGV)
  end
end # $RUBY_CHEMISTRY/show_electron_configuration.rb