#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ChemistryParadise::SplitMoleculeNames
#
# Input to this class here can be something like 'C12H11O11'. See the
# bottom of this file to find out what can be done.
#
# Do not input the charge of the molecule in question.
# =========================================================================== #
# require 'chemistry_paradise/split_molecule_names.rb'
# =========================================================================== #
require 'chemistry_paradise/base/base.rb'

module ChemistryParadise

class SplitMoleculeNames < ::ChemistryParadise::Base

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i = 'C12H11O11'
    )
    reset
    set_input(i)
    split # This is the actual run method.
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @result = []
    @total  = Hash.new(0) # Hash that keeps track of n elements each. Default value will be 0.
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result

  # ========================================================================= #
  # === is_number?
  #
  # This returns true if the input is a number, else false. For this it
  # uses the .to_i method trick, which returns 0 for non-numbers.
  # ========================================================================= #
  def is_number?(i)
    result = (i.to_i.to_s == i)
    return result
  end

  # ========================================================================= #
  # === split
  #
  # This method will return an array with all the elements.
  # ========================================================================= #
  def split(i = @input)
    array = []
    _ = ''.dup
    if i
      i.chars.each_with_index {|token, index|
        if is_number? token # We found a number here. We simply append it then.
          _ << token # return the old data
        elsif token.downcase == token
          _ << token
        else # Not a number, must be a character.
          unless _.empty?
            array << _
            _ = ''.dup
          end
          _ = token
        end
        array << _ if (index + 1) == i.size
      }
      @result = array
      determine_total
      return array
    end
  end

  # ========================================================================= #
  # === ignore_numbers
  #
  # This will ignore numbers in a guaranteed manner.
  # ========================================================================= #
  def ignore_numbers(i)
    return i.gsub(/\d+/,'')
  end

  # ========================================================================= #
  # === return_n_elements
  #
  # Must return an Integer.
  # ========================================================================= #
  def return_n_elements(i)
    result = 0
    if i =~ /\d+/ # Ok has at least one number.
      i = i[/(\d+)/][0]
    else
      i = 1
    end
    result = i.to_i
    return result
  end

  # ========================================================================= #
  # === determine_total
  #
  # If the input is @result then it was already properly pre-sorted for us.
  # ========================================================================= #
  def determine_total(i = @result)
    i.each {|entry|
      this_real_key = ignore_numbers(entry)
      if @total.has_key? this_real_key
        @total[this_real_key] = @total[this_real_key]+return_n_elements(entry)
      else # Else we must make a new key; the Laufindex kann aber 1-n sein.
        @total[this_real_key] = return_n_elements(entry)
      end
    }
  end

  # ========================================================================= #
  # === total?
  # ========================================================================= #
  def total?
    @total # This must always be a Hash.
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i)
    if i
      i = convert_parens(i) if i.include? ')'
      if i.include? ' '
        i.strip!
        if i.include? '+' # Ok, input here can be like: '2 Fe + 3 Cl2'
          i = i.gsub(/(\d)+ /,'\1')
          i = i.gsub(/ \+ /,'')
        end
        # ===================================================================== #
        # But it can also include internal ' ', which we will remove next.
        # ===================================================================== #
        i.delete!(' ')
      end
    end
    @input = i
  end

end

# =========================================================================== #
# === ChemistryParadise.split_this_molecular_formula_into_a_hash
# =========================================================================== #
def self.split_this_molecular_formula_into_a_hash(i)
  i = i.first if i.is_a? Array
  SplitMoleculeNames.new(i).result
end

end

if __FILE__ == $PROGRAM_NAME
  include ChemistryParadise
  puts 'Testing whether this class can properly Split Compound Names:'
  if ARGV.empty?
    pp SplitMoleculeNames.new.result
    pp SplitMoleculeNames.new('P2O5').result
    pp SplitMoleculeNames.new('C6H14N4O2').result
    pp SplitMoleculeNames.new('Al2(SO4)3').result
    pp SplitMoleculeNames.new('Fe(OH)3').result
    pp SplitMoleculeNames.new('H2SO4').result
    pp SplitMoleculeNames.new('Fe2(SO4)3').result
    pp SplitMoleculeNames.new('Fe2(SO4)3').total?
    p 'Will test this input next: 2 Fe + 3 Cl2'
    pp SplitMoleculeNames.new('2 Fe + 3 Cl2').result
  else
    SplitMoleculeNames.new(ARGV).result
  end
end # $CHEMISTRY/split_molecule_names.rb